function transImage = transformImage(I,M)
% function transImage = transformImage(I,M)
% transforms the image I by the homogeneous matrix M. It returns the
% transformed image

% calculate the most extreme points after transformation
[sy,sx] = size(I);
p1 = [1,1,1]';
p2 = [sx,1,1]';
p3 = [1,sy,1]';
p4 = [sx,sy,1]';

pH = M*[p1,p2,p3,p4];

% find the most extreme points after transformation
p = homo2euclidVectors(pH);
dim = int32(max(p'));

% create an array of that size - caveat: x and y must be flipped
transImage = zeros( dim([2 1]) );

%invert the transformation matrix for reverse computation
MM = M^-1;

% go along axes of the resulting image
if (dim(1)>0 && dim(2)>0)
    for n = 1:dim(2) %y
        for m = 1:dim(1); %x
            % transform the point (m,n)
            transP = MM* double([m,n,1]');
            eTransP  =  (homo2euclidVectors(transP) );
            x = eTransP(1);
            y = eTransP(2);
            if (x>1 && y>1 && x< sx-1 && y<sy-1)
                ix = 1-abs(x-int32(x));
                iy = 1-abs(y-int32(y));
           
                intx = int32(x); 
                inty = int32(y); 
                transImage(n,m) = ix*iy*I(inty,intx)+ix*(1-iy)*I(inty,intx+1) +(1-ix)*iy*I(inty+1,intx)  +(1-ix)*(1-iy)*I(inty+1,intx+1);
                %simple reverse transformation w/o interpolation
                %transImage(n,m) = I(inty, intx);
            end % if
        end; % for n
    end; % for m
end % if
